set echo on
set termout on
set feedback on
set linesize 120
set pagesize 500
set long 40000

cl scr


select ename
,      empno
,      sal
from   emp
/

pause

update emp
set    sal = sal * 1.15
where  job = 'CLERK' 
/
commit
/

select empno
,      ename
,      to_char(versions_starttime, 'DD-MON-YYYY HH24:MI') starttime
,      to_char(versions_endtime, 'DD-MON-YYYY HH24:MI') endtime
,      nvl(versions_operation,'I') versions_operation
,      sal
from   emp versions between timestamp minvalue and maxvalue
order 
by     empno, versions_starttime desc nulls last

pause

/


update emp
set    sal = sal + 115
where  empno = 7934
/

commit
/

select empno
,      ename
,      to_char(versions_starttime, 'DD-MON-YYYY HH24:MI') starttime
,      to_char(versions_endtime, 'DD-MON-YYYY HH24:MI') endtime
,      nvl(versions_operation,'I') versions_operation
,      sal
from   emp versions between timestamp minvalue and maxvalue
order 
by     empno, versions_starttime desc nulls last

pause

/

update emp
set    sal = sal * 1.06
where  empno = 7934

pause

/
commit
/



select empno
,      ename
,      nvl(versions_operation,'I') operation
,      sal
,      to_char(versions_starttime, 'DD-MON-YYYY HH24:MI') starttime
,      to_char(versions_endtime, 'DD-MON-YYYY HH24:MI') endtime
from   emp versions between timestamp minvalue and maxvalue
where  empno = 7934
order 
by     empno, versions_starttime desc nulls last

pause
/

rem find salary changes
select empno
       ,      ename
       ,      to_char(versions_starttime, 'DD-MON-YYYY HH24:MI') starttime
       ,      sal - lead(sal) over ( partition by empno
                                     order by versions_starttime
                                              desc nulls last
                                   ) salary_raise
       ,      sal new_sal                        
       ,      lead(sal) over ( partition by empno
                                     order by versions_starttime
                                              desc nulls last
                                   ) old_sal
       from   emp versions between timestamp minvalue and maxvalue
       order
       by     salary_raise desc nulls last

pause
/
       

rem finding the biggest salary raise       

select *
from ( select empno
       ,      ename
       ,      to_char(versions_starttime, 'DD-MON-YYYY HH24:MI') starttime
       ,      sal - lead(sal) over ( partition by empno
                                     order by versions_starttime
                                              desc nulls last
                                   ) salary_raise
       from   emp versions between timestamp minvalue and maxvalue
       order
       by     salary_raise desc nulls last
     )
where rownum = 1

pause

/


rem create and authorize dad as SCOTT

BEGIN
  DBMS_EPG.create_dad 
  ( dad_name => 'hrmhistory'
  , path => '/hrm/*'
  );
END;
/

begin
  DBMS_EPG.AUTHORIZE_DAD('hrmhistory');
end;
/ 

rem http://localhost:2100/hrm/scott.hrm_change_history.emp_history

pause

create or replace package HRM_CHANGE_HISTORY
is

procedure EMP_HISTORY;

end;
/


grant execute on HRM_CHANGE_HISTORY to public
/


pause

create or replace package body HRM_CHANGE_HISTORY
is

cursor c_emp_history
is
  select empno
  ,      ename
  ,      sal
  ,      versions_starttime
  ,      versions_endtime
  ,      nvl(versions_operation,'I') operation
  ,      lead(sal) over ( partition by empno
                                order by versions_starttime
                                         desc nulls last
                              ) old_salary
  from   emp versions between timestamp minvalue and maxvalue
  order
  by     versions_starttime desc nulls last
  ;

procedure write_emp_history
is
begin
  for r_emp in c_emp_history loop
     htp.p('<EMPCHANGE>');
     htp.p('<STARTTIME>'||r_emp.versions_starttime||'</STARTTIME>');
     htp.p('<ENDTIME>'||r_emp.versions_endtime||'</ENDTIME>');
     htp.p('<OPERATION>'||r_emp.operation||'</OPERATION>');
     htp.p('<EMPNO>'||r_emp.empno||'</EMPNO>');
     htp.p('<ENAME>'||r_emp.ename||'</ENAME>');
     htp.p('<SAL>'||r_emp.sal||'</SAL>');
     htp.p('<OLD_SAL>'||r_emp.old_salary||'</OLD_SAL>');
     htp.p('</EMPCHANGE>');
  end loop; -- r_remp
end write_emp_history;

procedure EMP_HISTORY
is
begin
  owa_util.mime_header('text/xml', FALSE); 
  owa_util.http_header_close; 
  htp.p('<?xml version="1.0" encoding="utf-8"?>');
  htp.p('<EMP_HISTORY>');  
  write_emp_history;
  htp.p('</EMP_HISTORY>');  
end emp_history;

end HRM_CHANGE_HISTORY;
/

pause


prompt http://localhost:2100/hrm/scott.hrm_change_history.emp_history

pause


rem more compact and elegant implementation using XMLTYPE


create or replace package body HRM_CHANGE_HISTORY
is

cursor c_emp_history_xml
is
  select xmlelement( "EMPHISTORY"
                   , xmlagg ( xmlelement( "EMPCHANGE"
                                        , xmlconcat ( xmlforest ( ename as "ENAME"
                                                                , empno as "EMPNO"
                                                                ,      sal as "SAL"
                                                                ,      versions_starttime as "STARTTIME"
                                                                ,      versions_endtime as "ENDTIME"
                                                                ,      nvl(versions_operation,'I') as "OPERATION"
                                                                ,      lead(sal) over ( partition by empno
                                                                                              order by versions_starttime
                                                                                                       desc nulls last
                                                                                            ) as "OLD_SAL"
                                                                )
                                                    )
                                        )
                            )
                   )
  from   emp versions between timestamp minvalue and maxvalue
  order
  by     versions_starttime desc nulls last
  ;


procedure EMP_HISTORY
is
  l_emp xmltype;                                                                                     
begin
  owa_util.mime_header('text/xml', FALSE); 
  owa_util.http_header_close; 
  htp.p('<?xml version="1.0" encoding="utf-8"?>');
  open c_emp_history_xml;
  fetch c_emp_history_xml into l_emp;
  close c_emp_history_xml;
  htp.p( l_emp.getStringVal);
end emp_history;

end HRM_CHANGE_HISTORY;
/

pause



rem as alternative to dbms_epg based PULL, implement push (create resource once, publish to HTTP, WEBDAV & HTTP clients)

DECLARE
  l_xml_dir VARCHAR2(30) := '/public/hrmData';
BEGIN

  IF dbms_xdb.CreateFolder(l_xml_dir) THEN
    dbms_xdb.setAcl(l_xml_dir,'/sys/acls/bootstrap_acl.xml');
    dbms_xdb.setAcl(l_xml_dir,'/sys/acls/all_owner_acl.xml');
    dbms_xdb.setAcl(l_xml_dir,'/sys/acls/ro_all_acl.xml');
    dbms_output.put_line('Success');
  ELSE
    dbms_output.put_line('FAILED!');  
  END IF;
  
  COMMIT;

END;
/




create or replace package HRM_CHANGE_HISTORY
is

procedure EMP_HISTORY;
procedure PUBLISH_EMP_HISTORY;

end HRM_CHANGE_HISTORY;
/


rem when ACCESS DENIED, see http://forums.oracle.com/forums/thread.jspa?messageID=2599375
rem EXEC DBMS_XDB.setacl(res_path => '/public/hrmData' , acl_path => '/sys/acls/all_all_acl.xml');


create or replace package body HRM_CHANGE_HISTORY
is

cursor c_emp_history_xml
is
  select xmlelement( "EMPHISTORY"
                   , xmlagg ( xmlelement( "EMPCHANGE"
                                        , xmlconcat ( xmlforest ( ename as "ENAME"
                                                                , empno as "EMPNO"
                                                                ,      sal as "SAL"
                                                                ,      versions_starttime as "STARTTIME"
                                                                ,      versions_endtime as "ENDTIME"
                                                                ,      nvl(versions_operation,'I') as "OPERATION"
                                                                ,      lead(sal) over ( partition by empno
                                                                                              order by versions_starttime
                                                                                                       desc nulls last
                                                                                            ) as "OLD_SAL"
                                                                )
                                                    )
                                        )
                            )
                   )
  from   emp versions between timestamp minvalue and maxvalue
  order
  by     versions_starttime desc nulls last
  ;


procedure EMP_HISTORY
is
  l_emp xmltype;                                                                                     
begin
  owa_util.mime_header('text/xml', FALSE); 
  owa_util.http_header_close; 
  htp.p('<?xml version="1.0" encoding="utf-8"?>');
  open c_emp_history_xml;
  fetch c_emp_history_xml into l_emp;
  close c_emp_history_xml;
  htp.p( l_emp.getStringVal);
end emp_history;

procedure PUBLISH_EMP_HISTORY
is
  l_xml_dir VARCHAR2(30) := '/public/hrmData/';
  l_doc_name varchar2(30):= 'EmpChangeHistory.xml';
  l_res BOOLEAN;
  l_emp xmltype;                                                                                     
begin
  open c_emp_history_xml;
  fetch c_emp_history_xml into l_emp;
  close c_emp_history_xml;
  if DBMS_XDB.EXISTSRESOURCE(abspath => l_xml_dir||l_doc_name)
  then
    dbms_xdb.deleteresource(abspath => l_xml_dir||l_doc_name);
  end if;
  l_res := dbms_xdb.createResource
           ( abspath=> l_xml_dir||l_doc_name
           , data=> l_emp
           );
  COMMIT;
end PUBLISH_EMP_HISTORY;

end HRM_CHANGE_HISTORY;
/

pause

prompt look in http://localhost:2100/public/hrmData



BEGIN 
  HRM_CHANGE_HISTORY.PUBLISH_EMP_HISTORY; 
END;
/


rem access xml doc

prompt browser:  http://localhost:2100/public/hrmData

rem sql: 

SELECT xdburitype('/public/hrmData/EmpChangeHistory.xml').getClob()
FROM   dual

pause

/



rem Open Explorer. Go to My Network Places and run the Add Network Place wizard. 
rem On the first screen, just hit Next. Highlight "Choose another network location" and hit next.
rem In the "Internet or network address" field, enter url for public folder :  http://localhost:2100/public/
rem You should get a connect dialog asking for a user name and password. select OK. Give it a name or accept the default. Hit next.
rem Make sure the "open this network place" is checked and hit finish. 
rem You should now see the new folder in your network places. Double click on that and then double click on the public folder. 
rem It will probably ask you to enter a user name and password again. Use scott/tiger again.
