set echo on
set termout on
set feedback on

set pagesize 500
set long 40000

cl scr


select *
from   ( select ename
         ,      sal
         ,      hiredate
         ,        2 * nvl( emp_sal_around_1500.score, 0)
                + nvl(emp_hiredate_1982.score, 0)  score
         from   emp e
                left outer join
                ( select case
                         when sal < 1500
                         then case
                              when sal > 1000
                     then (sal - 1000)/(1500 - 1000)
                     else 0
                     end
                when sal < 2000
                then (sal - 2000)/(1500 - 2000)
                else 0
                end  score
                  from   emp
                ) emp_sal_around_1500
                on (e.rowid = emp_sal_around_1500.rowid)
                left outer join
                ( select case extract (year from hiredate)
                         when 1982 
                         then 1
                         when 1983 
                         then 0.5
                         end  score 
                  from   emp
                ) emp_hiredate_1982
                on (e.rowid = emp_hiredate_1982.rowid)
       order
       by     score desc
       )
where rownum < 6
         
pause

/


pause

create or replace function number_range_score
( value in number
, target in number
, low_end in number default null -- pass in target if there is no scoring range below the target value; null means any value lower than target has a full score
, high_end in number default null -- pas in target if there is no scoring range over the target value; null means any value higher than target has a full score
) return number
 is
begin
  return
  case
  when value = target
  then 1
  when value < target
  then case
       when low_end is null
       then 1
       when value > low_end
       then (value-low_end)/(target - low_end )
       else 0
       end
  when value > target
  then case
       when high_end is null
       then 1
       when value > high_end
       then 0
       else (value - high_end)/(target - high_end)
       end
  else 0
  end  ;
end number_range_score;
/

pause


select *
from   ( select ename
         ,      sal
         ,      hiredate
         ,        2 * number_range_score(sal, 1500, 1000, 2000)
                + number_range_score(extract (year from hiredate), 1982, 1982, 1983)  
                score
         from   emp e
         order
         by     score desc
       )
where rownum < 6

pause

/

pause


create type query_match_score_t as object
( score number(20,10)
, weight number(5,2)
, criteria varchar2(2000)
)
/

create type query_match_score_table_t as 
table of query_match_score_t
/



select empno
,      query_match_score_t( number_range_score(sal, 1500, 1000, 2000), 2
                          , 'Salary must be as close to 1500 as possible, and at least between 1000 and 2000') score
from   emp

pause

/


select *
from (
  select empno
  ,      sum(scores.match_score.score * scores.match_score.weight) weighted_score
  ,      cast(collect(scores.match_score) as query_match_score_table_t) criteria
  from   (  select empno
            ,      query_match_score_t( number_range_score(sal, 1500, 1000, 2000), 2
                                      , 'Salary must be as close to 1500 as possible, and at least between 1000 and 2000') match_score
            from   emp
         ) scores
  group
  by     empno
  order
  by     weighted_score desc
  )
where rownum < 6

pause

/

pause

select *
from (
       select empno
       ,      sum(score * weight) weighted_score
       ,      cast(collect(query_match_score_t( score, weight,criteria)) as query_match_score_table_t) criteria
       from   (
                select empno
                ,      number_range_score(sal, 1500, 1000, 2000) score
                ,      2 weight
                ,      'Salary must be as close to 1500 as possible, and at least between 1000 and 2000'  criteria
                from   emp
                union all
                select empno
                ,      number_range_score(extract (year from hiredate), 1982, 1982, 1983) score
                ,      1 weight
                ,      'Employee is ideally hired in 1982; a hiredate in 1983 is also somewhat valuable'
                from   emp
              )
       group
       by     empno
       order
       by     weighted_score desc
     )
where rownum < 6

pause

/

pause

rem slightly nicer format of the search results

select empno
,      (select ename||' - '||job||' - '||hiredate||' - '||sal from emp where empno = scores.empno) details
,      weighted_score 
,      ( select cast(collect( substr(criteria,1,105)||': '||score*100||'% ('||weight||')') as string_table) 
         from table(criteria) 
         where score > 0) criteria
from   ( select empno
         ,      sum(score * weight) weighted_score
         ,      cast(collect(query_match_score_t( score, weight,criteria)) as query_match_score_table_t) criteria
         from   (
                 select empno
                 ,      number_range_score(sal, 1500, 1000, 2000) score
                 ,      2 weight
                 ,      'Salary must be as close to 1500 as possible, and at least between 1000 and 2000'  criteria
                 from   emp
                 union all
                 select empno
                 ,      number_range_score(extract (year from hiredate), 1982, 1982, 1983) score
                 ,      1 weight
                 ,      'Employee is ideally hired in 1982; a hiredate in 1983 is also somewhat valuable'
                 from   emp
                )
         group
         by     empno
         order
         by     weighted_score desc
       ) scores
where  rownum < 6

pause

/


rem see how easy it is to add a fuzzy search criterium
rem weight = 1.5; we would prefer clerks but can live with ANALYSTS (60%) and SALESMEN (40%)

pause

select empno
,      case job when 'CLERK' then 1 when 'ANALYST' then 0.6 when 'SALESMAN' then 0.4 end score
,      1.5 weight
,      'We prefer CLERKs; ANALYST scores 60% and SALESMAN is 40%'
from   emp
/

pause



select empno
,      (select ename||' - '||job||' - '||hiredate||' - '||sal from emp where empno = scores.empno) details
,      weighted_score 
,      ( select cast(collect( substr(criteria,1,105)||': '||score||' ('||weight||')') as string_table) 
         from table(criteria) 
         where score > 0) criteria
from   ( select empno
         ,      sum(score * weight) weighted_score
         ,      cast(collect(query_match_score_t( score, weight,criteria)) as query_match_score_table_t) criteria
         from   (
                 select empno
                 ,      number_range_score(sal, 1500, 1000, 2000) score
                 ,      2 weight
                 ,      'Salary must be as close to 1500 as possible, and at least between 1000 and 2000'  criteria
                 from   emp
                 union all
                 select empno
                 ,      number_range_score(extract (year from hiredate), 1982, 1982, 1983) score
                 ,      1 weight
                 ,      'Employee is ideally hired in 1982; a hiredate in 1983 is also somewhat valuable'
                 from   emp
                 union all
                 select empno
                 ,      case job when 'CLERK' then 1 when 'ANALYST' then 0.6 when 'SALESMAN' then 0.4 end  score
                 ,      1.5 weight
                 ,      'We prefer CLERKs; ANALYST scores 60% and SALESMAN is 40%'
                 from   emp
                )
         group
         by     empno
         order
         by     weighted_score desc
       ) scores
where  rownum < 6

pause

/

